#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.widgets.grids import ShadedRect
from reportlab.graphics.shapes import Drawing, _DrawingEditorMixin, String
from reportlab.lib.colors import grey

class Graphic(_DrawingEditorMixin,Drawing):
    def __init__(self,width=400,height=200,*args,**kw):
        Drawing.__init__(self,width,height,*args,**kw)
        self.width       = 72
        self.height      = 20
        self._add(self,ShadedRect(),name='sr',validate=None,desc=None)
        self.sr.strokeColor    = None
        self.sr.fillColorEnd   = grey
        self.sr.width=self.width
        self.sr.height=self.height
        self._add(self,String(self.width*0.5,6,'text', textAnchor='middle', fontName='Helvetica-Bold',fontSize=12),name='s',validate=None,desc=None)

if __name__=="__main__": #NORUNTESTS
    Graphic().save(formats=['pdf'],outDir='.',fnRoot=None)
